% function [alpha,H] = apes(x, L, om, [method])
%
% Calculates the APES estimates of the complex amplitude
% alpha from the data x in the frequency points
% specified by the vector om. The APES filter has the
% length L.
% The variables are assumed to satisfy the following properties:
%   Name:      Size:     Constraint:          Meaning:
%-> x          1xN/Nx1   complex              signal
%-> L          1x1       >0,integer           length of the apes filter
%-> om         1xNom/Nomx1 real               vector of angular frequencies
%-> method     string    'unbiased' or        use forward or forward-backward
%                        'forback'            estimate of the covariance matrix
%<- alpha      Nomx1     complex              vector of complex amplitude estimates
%                                             pertaining to the frequencies
%                                             in om assuming a start index
%                                             of 0
%<- H          LxNom     complex              matrix containing the APES
%                                             filter coefficents pertaining
%                                             to the frequencies in om
% Created:        2008-12-04  by Jesper Kjær
% Last modified:  2011-10-25  by Jesper Kjær
function [alpha,H] = apes(x, L, om, method)
  if nargin<4,
      method = 'unbiased';
  end
% Initiate.    
  N  = length(x); %length of observed data vector
  M  = N-L+1		;	%number of samples to average over
 
% Compute the spectral estimate for each frequncy point in om
  X   = flipud(hankel(x(1:L), x(L:N))); %data matrix
  C = (X*X')/M;
  if strcmp(method,'unbiased')
    iC = C\eye(L)               ; %forward estimate of inverse covariance matrix
  else
    iC = 2*((C+rot90(C,2).')\eye(L)); %forward-backward estimate of inverse covariance matrix
  end
  Nom	= length(om) 							; %length of frequency vector
  alpha = zeros(Nom,1)					; %initilise estimation vector of complex amplitude
  H = zeros(L,Nom)              ; %initilise matrix for saving the APES filtercoefficients
  for ii= 1:Nom
    a  = exp(-1i*om(ii)*(0:L-1)')				  ; %calculate Fourier vector for current f
    g = X * exp(-1i*om(ii)*(L-1:N-1)')/M      ; %calculate temporary vector zw
    %calculate inverse corrected covariance matrix
    if strcmp(method,'unbiased') % forward
      iQ = iC + iC*(g*g')*iC / (1-g'*iC*g);
    else % forward-backward
      G = [g conj(flipud(g))]/sqrt(2);
      iQ = iC + iC*G*((eye(2)-G'*iC*G)\(G'*iC));
    end
    H(:,ii) = iQ*a/(a'*iQ*a)              ; %calculate the APES filter
    alpha(ii) = H(:,ii)'*g                ; %calculate the APES estimate
  end
  
